#if 0
/*
 * (c) 2017 Petr Ročkai <code@fixp.eu>
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#include <divine/vm/crash.hpp>
#include <divine/vm/pointer.hpp>
#include <divine/vm/context.hpp>
#include <divine/vm/program.hpp>
#include <divine/vm/eval.hpp>

namespace divine::vm::dbg
{

template< typename Context >
void Core< Context >::dump( Context &offending )
{
    return; /* TBD */

    static std::atomic_flag _done = ATOMIC_FLAG_INIT;
    if ( _done.test_and_set() )
        return;

    std::stringstream fn;
    fn << "divm_core.";
    fn << getpid();
    std::ofstream core( fn.str() );

    std::cerr << "DiVM crashed, writing " << fn.str() << ", ctx = "
              << &offending << "... " << std::flush;

    auto &heap = offending.heap();

    auto write_obj = [&]( int obj, auto internal )
    {
        HeapPointer ptr( obj, 0 );
        auto size = heap.size( ptr, internal );

        core << std::hex << obj << " " << heap.size( ptr, internal ) << std::endl;
        core.write( heap.template unsafe_deref< char >( ptr, internal ), size );
        core << std::endl;
    };

    core << "register values" << std::endl;
    for ( int i = 0; i < _VM_CR_Last; ++i )
        core << std::hex << i << " " << offending._reg[ i ].integer << std::endl;

    core << "snapshot data" << std::endl;
    for ( auto si = heap.snap_begin(); si != heap.snap_end(); ++si )
        write_obj( si->first, si->second );

    core << "exception data" << std::endl;
    for ( auto except : heap._l.exceptions )
        write_obj( except.first, except.second );

    std::cerr << "done" << std::endl;
}

template struct Core< vm::Context< Program, mem::CowHeap > >;
template struct Core< vm::Context< Program, MutableHeap > >;
template struct Core< vm::Context< Program, SmallHeap > >; /* Program::Context */
template struct Core< vm::Context< t_vm::TProgram, SmallHeap > >;

}
#endif
