// -*- mode: C++; indent-tabs-mode: nil; c-basic-offset: 4 -*-

/*
 * (c) 2017 Petr Ročkai <code@fixp.eu>
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#if 0
#pragma once

#include <brick-bitlevel>
#include <divine/vm/xg-type.hpp>
#include <divine/vm/xg-code.hpp>
#include <divine/vm/lx-loader.hpp>

/*
 * LX generation goes like this:
 *
 * - assign addresses to basic blocks
 * - assign addresses to slots
 * - compute constants (consulting the above assignments)
 * - generate the image for initial mutable globals
 * - generate the instruction stream
 *
 * The runtime representation has the following components:
 *
 * - type table          (constant item size)
 * - function table      (likewise)
 * - reversed slot table (ditto)
 * - [constants size, type table size]
 *                                     <---- _VM_CR_Constants points here
 * - constant data
 * - code
 *
 * - mutable data
 *
 * The first four components are immutable and live in a single heap object
 * together. Offsets and sizes for each of the four sections are given at the
 * beginning of that heap object. The last one is mutable and gets loaded into
 * a separate heap object, without further metadata.
 *
 * Whenever _VM_CR_Constants is modified, the type/slot and code base pointers
 * are recomputed. While _VM_CR_Constants must point at the constant data area
 * (i.e. it has non-zero offset), this *must* be within an object that also
 * contains the remaining sections.
 *
 * Slots are indexed negatively from _VM_CR_Constants, while types and
 * functions are indexed positively from the start of the heap object (offset
 * 0) and from the start + type table size respectively.
 */

namespace divine::vm::xg
{

struct Driver
{
    llvm::Module *_module;
    llvm::DataLayout TD;
    MutableHeap _heap;

    Types _types;
    AddressMap _addr;

    template< typename Heap >
    lx::Image emit( Heap &h )
    {
        _addr.build( _module );
        // int offset = 0;
        /* offset += */ _types.emit( h );
    }

    Driver( llvm::Module *m ) : TD( m ), _types( m ) { NOT_IMPLEMENTED(); }
};

}
#endif
